//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useState, useEffect, useRef } from "react";
import { cloneDeep, each, find, isEmpty } from "lodash";
import { isNonEmptyString, newGuid, isObject, insertTreeItem, updateTreeItem, getTreeItem, removeTreeItem, getEntity, formatText, doNothing } from "primeobjects-helper-util";
import { useSolution, usePrimaryColor } from "primeobjects-helper-ui";
import { callApi } from "../../context/auth-call-api";
import Popconfirm from "../../controls/antd/popconfirm";
import Dropdown from "../../controls/antd/dropdown";
import Menu from "../../controls/antd/menu";
import CSS from "primeobjects-ui-web-store/build/cjs/css";
import { SVG, getLocalStorage, setLocalStorage } from "primeobjects-ui-web-basic";
import Button from "../../controls/action-button";
import { getApiEndpoint, getGlobalComponent } from "../../util";
var TREE_HEAD_CSS = "\n    .tree-header .ant-select\n    {\n        height: 32px;\n        min-width: 120px;\n    }\n\n    .tree-header .ant-select-selector\n    {\n        padding: 0 !important;\n        line-height: 1;\n        font-weight: bold !important;\n        font-size: 1.2rem;\n        padding-right: 32px !important;\n    }\n\n    .tree-header .ant-select-selection-item\n    {\n        font-weight: 500 !important;\n        font-size: 1.2rem;\n        padding-right: 32px !important;\n    }\n\n    .tree-header .ant-select-arrow\n    {\n        height: 1.2rem;\n    }\n";
var SingleLineInputField = undefined;
var TreeField = undefined;
export var ListTreeSection = function (props) {
    var _a, _b;
    if (!TreeField) {
        console.log("Get global components in /sections/list/tree.tsx");
        SingleLineInputField = getGlobalComponent("SingleLineInputField");
        TreeField = getGlobalComponent("TreeField");
    }
    var entityName = props.entityName, attributeName = props.attributeName, regardingEntityType = props.regardingEntityType, regardingEntityName = props.regardingEntityName, organizationId = props.organizationId, readonly = props.readonly, showFilterButton = props.showFilterButton;
    var solution = useSolution();
    var _c = useState(undefined), entity = _c[0], setEntity = _c[1];
    var uiName = entity && entity.uiName;
    var uiCollectionName = entity && entity.uiCollectionName;
    var wrapperClassName = props.wrapperClassName
        ? props.wrapperClassName
        : "";
    var wrapperStyle = props.wrapperStyle ? props.wrapperStyle : {};
    var primaryColor = usePrimaryColor();
    var _d = useState([]), expendedIds = _d[0], setExpendedIds = _d[1];
    var _e = useState([]), checkedIds = _e[0], setCheckedIds = _e[1];
    var _f = useState([]), checkedNodes = _f[0], setCheckedNodes = _f[1];
    var _g = useState(undefined), treeData = _g[0], setTreeData = _g[1];
    var _h = useState(""), error = _h[0], setError = _h[1];
    var _j = useState("Loading ".concat(uiCollectionName, " ...")), doing = _j[0], setDoing = _j[1];
    var _k = useState(""), selectedId = _k[0], setSelectedId = _k[1];
    var _l = useState(undefined), selectedNode = _l[0], setSelectedNode = _l[1];
    var _m = useState(undefined), filteredNode = _m[0], setFilteredNode = _m[1];
    var _o = useState(""), op = _o[0], setOp = _o[1];
    var _p = useState(""), text = _p[0], setText = _p[1];
    var abortController = useRef();
    var cacheKey = "list-tree-".concat(entityName, "-").concat(regardingEntityName, "-").concat(regardingEntityType, "-").concat(organizationId, "-").concat(solution.id);
    doNothing({ selectedNode: selectedNode, checkedNodes: checkedNodes });
    useEffect(function () {
        if (isNonEmptyString(entityName)) {
            setEntity(getEntity(solution, entityName));
        }
    }, [entityName]);
    var refreshData = function (quite) {
        var _a;
        if (!quite) {
            setTreeData(undefined);
            setDoing("Loading ".concat(uiCollectionName, " ..."));
            setError("");
            setSelectedId("");
        }
        (_a = abortController.current) === null || _a === void 0 ? void 0 : _a.abort();
        abortController.current = new AbortController();
        callApi(solution, getApiEndpoint(solution, "organization", "retrieve-data", entityName), {
            entityName: entityName,
            regardingEntityName: regardingEntityName,
            regardingEntityType: regardingEntityType,
            organizationId: organizationId
        }, "GET", {
            abortSignal: abortController.current.signal
        })
            .then(function (result) {
            var data = {
                entityName: entityName,
                regardingEntityName: regardingEntityName,
                regardingEntityType: regardingEntityType,
                data: []
            };
            if (!isEmpty(result)) {
                data = result;
            }
            if (!quite ||
                (quite && JSON.stringify(treeData) != JSON.stringify(data))) {
                setLocalStorage(cacheKey, data);
                setTreeData(data);
            }
        })
            .catch(function (error) {
            if ((error === null || error === void 0 ? void 0 : error.message) != "canceled") {
                console.error(error);
                setError("Failed to retrieve ".concat(uiCollectionName, "."));
            }
        })
            .finally(function () {
            setDoing("");
        });
    };
    useEffect(function () {
        if (entity) {
            //try to get from local storage
            var data = getLocalStorage(cacheKey);
            if (data) {
                refreshData(true);
            }
            else {
                refreshData(false);
            }
        }
    }, [
        entity,
        regardingEntityType,
        regardingEntityName,
        attributeName,
        organizationId
    ]);
    useEffect(function () {
        setFilteredNode(undefined);
        setSelectedNode(undefined);
        setSelectedId("");
    }, [props.removeFilteredNode]);
    useEffect(function () {
        setCheckedNodes(props.checkedNodes ? props.checkedNodes : []);
    }, [props.checkedNodes]);
    var onChange = function (newData) {
        if (!treeData)
            return;
        setDoing("Updating ".concat(uiCollectionName, " ..."));
        callApi(solution, getApiEndpoint(solution, "data", treeData.id ? "update" : "create", entityName), { data: __assign(__assign({}, treeData), { data: newData }) }, treeData.id ? "PUT" : "POST")
            .then(function (result) {
            setLocalStorage(cacheKey, result);
            setTreeData(result);
        })
            .catch(function (error) {
            console.error(error);
            setError("Failed to save ".concat(uiCollectionName, "."));
        })
            .finally(function () {
            setDoing("");
            setOp("");
        });
    };
    var onClickAddNode = function (type) {
        setText("");
        setOp("add-".concat(type));
    };
    // const getFilterNodes = (): Record<string, any>[] => {
    //     if (isArray(checkedNodes) && checkedNodes.length > 0) {
    //         return checkedNodes;
    //     } else {
    //         if (selectedNode) return [selectedNode];
    //     }
    //     return [];
    // };
    // const onClickFilter = () => {
    //     const filterNodes = getFilterNodes();
    //     props.onClickFilter && props.onClickFilter(filterNodes);
    // };
    var onClickFilterButton = function (newSelectedNode) {
        setCheckedIds([]);
        setCheckedNodes([]);
        if ((filteredNode === null || filteredNode === void 0 ? void 0 : filteredNode.id) == newSelectedNode.id) {
            setFilteredNode(undefined);
            props.onClickFilter &&
                props.onClickFilter([
                    __assign(__assign({}, newSelectedNode), { entityName: entityName, attributeName: attributeName, remove: true })
                ]);
        }
        else {
            setFilteredNode(newSelectedNode);
            props.onClickFilter &&
                props.onClickFilter([
                    __assign(__assign({}, newSelectedNode), { entityName: entityName, attributeName: attributeName })
                ]);
        }
    };
    var onClickRefresh = function () {
        refreshData(false);
    };
    var onClickSubmit = function (type) {
        if (!treeData)
            return;
        var newId = newGuid();
        var newData = cloneDeep(treeData.data ? treeData.data : []);
        switch (type) {
            case "add-root": {
                if (!isNonEmptyString(text))
                    return;
                if (!find(newData, function (i) { var _a; return ((_a = i.text) === null || _a === void 0 ? void 0 : _a.toLowerCase()) == text.toLowerCase(); })) {
                    onChange(__spreadArray([{ id: newId, text: text }], newData, true));
                }
                break;
            }
            case "add-above": {
                if (!isNonEmptyString(text))
                    return;
                onChange(insertTreeItem(newData, selectedId, { id: newId, text: text }, "above"));
                break;
            }
            case "add-below": {
                if (!isNonEmptyString(text))
                    return;
                onChange(insertTreeItem(newData, selectedId, { id: newId, text: text }, "below"));
                break;
            }
            case "add-children": {
                if (!isNonEmptyString(text))
                    return;
                onChange(insertTreeItem(newData, selectedId, { id: newId, text: text }, "children"));
                setExpendedIds(__spreadArray(__spreadArray([], expendedIds, true), [selectedId], false));
                break;
            }
            case "delete": {
                onChange(removeTreeItem(newData, selectedId));
                break;
            }
            case "edit": {
                if (!isNonEmptyString(text))
                    return;
                onChange(updateTreeItem(newData, selectedId, function (item) {
                    return __assign(__assign({}, item), { text: text });
                }));
                break;
            }
        }
        setText("");
        setOp("");
    };
    var onDrop = function (newData) {
        onChange(newData ? newData : []);
    };
    var onExpand = function (newExpendedIds) {
        setExpendedIds(newExpendedIds ? newExpendedIds : []);
    };
    var onCheck = function (newCheckedIds) {
        if (!treeData)
            return;
        setCheckedIds(newCheckedIds ? newCheckedIds : []);
        var checkedNodes = [];
        each(newCheckedIds, function (newCheckedId) {
            var newCheckedNode = getTreeItem(treeData.data, function (item) { return (item === null || item === void 0 ? void 0 : item.id) == newCheckedId; });
            if (newCheckedNode)
                checkedNodes.push(__assign(__assign({}, newCheckedNode), { entityName: entityName, attributeName: attributeName }));
        });
        setCheckedNodes(checkedNodes);
    };
    var onSelect = function (newSelectIds) {
        if (!treeData)
            return;
        var nodeBeingClicked = undefined;
        if (!(newSelectIds && newSelectIds.length > 0)) {
            nodeBeingClicked = selectedNode;
            setSelectedNode(undefined);
            setSelectedId("");
        }
        else {
            var newSelectedId_1 = newSelectIds[0];
            var node = getTreeItem(treeData.data, function (item) { return (item === null || item === void 0 ? void 0 : item.id) == newSelectedId_1; });
            nodeBeingClicked = __assign(__assign({}, node), { entityName: entityName, attributeName: attributeName });
            setSelectedNode(nodeBeingClicked);
            setSelectedId(newSelectedId_1);
        }
        setOp("");
        setText("");
        if (!showFilterButton && nodeBeingClicked) {
            onClickFilterButton(nodeBeingClicked);
        }
    };
    var onClickEdit = function () {
        if (!treeData)
            return;
        var newSelected = getTreeItem(treeData.data, function (item) { return (item === null || item === void 0 ? void 0 : item.id) == selectedId; });
        setText(isObject(newSelected) ? newSelected === null || newSelected === void 0 ? void 0 : newSelected.text : "");
        setOp("edit");
    };
    var checkIsFilteredItem = function (item) {
        return (filteredNode === null || filteredNode === void 0 ? void 0 : filteredNode.id) == item.id;
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(CSS, { id: "".concat(uiCollectionName, "-header-css"), content: TREE_HEAD_CSS }),
        React.createElement("div", { className: "w-full h-full flex flex-col overflow-x-hidden bg-white ".concat(wrapperClassName), style: wrapperStyle },
            React.createElement("div", { className: "tree-header flex flex-row p-4 border-b items-center", style: { minHeight: 65 } },
                React.createElement("h2", { className: "text-lg font-semibold leading-none m-0" }, formatText(uiCollectionName, "capital-first")),
                !isNonEmptyString(doing) && (React.createElement("div", { className: "flex-1 flex flex-row" },
                    React.createElement("div", { className: "flex-1" }),
                    React.createElement(Button, { id: "".concat(uiName, "-refresh"), iconSrc: "/icons/platform/refresh.svg", color: primaryColor, onClick: onClickRefresh, tooltipText: "Refresh", tooltipMouseEnterDelay: 0, tooltipMouseLeaveDelay: 0 }),
                    isNonEmptyString(selectedId) && !readonly && (React.createElement(Popconfirm, { placement: "bottom", title: "Delete the selected ".concat(uiName, "?"), onConfirm: function () { return onClickSubmit("delete"); }, okText: "Delete", okType: "danger", cancelText: "Cancel" },
                        React.createElement(Button, { id: "".concat(uiName, "-delete-node"), iconSrc: "/icons/platform/node/delete-node.svg", color: primaryColor, onClick: onClickEdit, tooltipText: "Delete the selected ".concat(uiName), tooltipMouseEnterDelay: 0, tooltipMouseLeaveDelay: 0 }))),
                    isNonEmptyString(selectedId) && !readonly && (React.createElement(Button, { id: "".concat(uiName, "-edit-node"), iconSrc: "/icons/platform/node/edit-node.svg", color: primaryColor, onClick: onClickEdit, tooltipText: "Edit the selected ".concat(uiName), tooltipMouseEnterDelay: 0, tooltipMouseLeaveDelay: 0 })),
                    isNonEmptyString(selectedId) && !readonly && (React.createElement(Dropdown, { trigger: ["click"], placement: "top", overlay: React.createElement(Menu, null,
                            React.createElement(Menu.Item, { onClick: function () {
                                    return onClickAddNode("above");
                                } }, "Above the selected"),
                            React.createElement(Menu.Item, { onClick: function () {
                                    return onClickAddNode("below");
                                } }, "Below the selected"),
                            React.createElement(Menu.Item, { onClick: function () {
                                    return onClickAddNode("children");
                                } }, "As a children")) },
                        React.createElement(Button, { id: "".concat(uiName, "-add-node"), iconSrc: "/icons/platform/node/add-node.svg", color: primaryColor, tooltipPlacement: "right", tooltipText: "Add new ".concat(uiName), tooltipMouseEnterDelay: 0, tooltipMouseLeaveDelay: 0 }))),
                    !isNonEmptyString(selectedId) && !readonly && (React.createElement(Button, { id: "".concat(uiName, "-new-node"), iconSrc: "/icons/platform/node/add-node.svg", color: primaryColor, onClick: function () { return onClickAddNode("root"); }, tooltipPlacement: "right", tooltipText: "Add new ".concat(uiName), tooltipMouseEnterDelay: 0, tooltipMouseLeaveDelay: 0 })),
                    props.onClickClose && (React.createElement(Button, { id: "".concat(uiName, "-close"), iconSrc: "/icons/platform/x.svg", color: primaryColor, onClick: props.onClickClose, tooltipText: "Close", tooltipMouseEnterDelay: 0, tooltipMouseLeaveDelay: 0 }))))),
            (op.indexOf("add-") >= 0 || op == "edit") && (React.createElement("div", { className: "w-full flex flex-row py-2 items-center py-6 px-4 border-b border-gray-100 bg-white", style: { height: 65 } },
                React.createElement(SingleLineInputField, { id: "".concat(uiName, "-input-name"), inputWrapperStyle: { borderWidth: 0 }, inputStyle: {
                        height: 30,
                        background: "transparent",
                        padding: 0
                    }, format: "text", onChange: function (v) {
                        setText(v ? v : "");
                    }, placeholder: "Type ".concat(uiName, " here"), value: text }),
                React.createElement("button", { style: { height: 24 }, className: "cursor-pointer inline-flex items-center self-center justify-center py-2 px-1 rounded shadow hover:shadow-lg font-medium bg-gray-50", onClick: function () { return setOp(""); } },
                    React.createElement(SVG, { src: "/icons/platform/x.svg", style: { width: 16 }, color: "#333333" })),
                React.createElement("button", { style: { height: 24 }, className: "cursor-pointer inline-flex ml-2 items-center self-center justify-center py-2 px-1 rounded shadow hover:shadow-lg font-medium bg-sky-50", onClick: function () { return onClickSubmit(op); } },
                    React.createElement(SVG, { src: "/icons/platform/checkmark.svg", style: { width: 16 }, color: "#333333" })))),
            isNonEmptyString(doing) && (React.createElement("div", { className: "w-full flex p-4" },
                React.createElement(SVG, { src: "/icons/platform/loading.svg", className: "spinner", style: { width: 22, height: 22 } }),
                React.createElement("span", { className: "pl-2" }, "Loading ..."))),
            isNonEmptyString(error) && !isNonEmptyString(doing) && (React.createElement("div", { className: "w-full flex p-4" },
                React.createElement("span", { className: "text-base text-red-700" }, error))),
            !isNonEmptyString(error) &&
                !isNonEmptyString(doing) &&
                ((_a = treeData === null || treeData === void 0 ? void 0 : treeData.data) === null || _a === void 0 ? void 0 : _a.length) == 0 && (React.createElement("div", { className: "w-full flex p-4" },
                React.createElement("span", { className: "text-base text-gray-700" }, "There is no ".concat(entity === null || entity === void 0 ? void 0 : entity.uiName, ".")))),
            !isNonEmptyString(error) &&
                !isNonEmptyString(doing) &&
                ((_b = treeData === null || treeData === void 0 ? void 0 : treeData.data) === null || _b === void 0 ? void 0 : _b.length) > 0 && (React.createElement("div", { className: "h-full flex flex-col w-full overflow-hidden bg-white p-4" },
                React.createElement(TreeField, { checkable: props.supportAdvancedFilter, wrapperClassName: "h-full", inputWrapperClassName: "h-full", id: "".concat(entityName, "-").concat(regardingEntityName, "-").concat(regardingEntityType, "-tree"), disabled: isNonEmptyString(doing), value: treeData === null || treeData === void 0 ? void 0 : treeData.data, expendedIds: expendedIds, checkedIds: checkedIds, selectedIds: selectedId ? [selectedId] : [], onExpand: onExpand, onCheck: onCheck, onDrop: onDrop, onSelect: onSelect, actionButton: showFilterButton
                        ? {
                            iconSrc: "/icons/platform/filter.svg",
                            onClick: onClickFilterButton,
                            checkIsActionItem: checkIsFilteredItem
                        }
                        : undefined }))))));
};
export default ListTreeSection;
//# sourceMappingURL=tree.js.map